# -*- coding: UTF-8 -*-

import MySQLdb
import nes_common.logger as logger
from mysql.nes_exec import Exec
from nes_command import Command
from nes_common.config import config
from nes_common.inserter import Inserter


class DB_Inserter(Inserter):
    dbVersionIsOK = {}

    def __init__(self, shop, loadDictStatusSender=None):
        self.log = logger.getLogger("Shop", shop.get('shopId', ''))
        host = config.getString('mysql', 'host')
        port = config.getInt('mysql', 'port')
        user = config.getString('mysql', 'user')
        password = config.getString('mysql', 'password')
        self.migrateVersion = config.getInt('mysql', 'migrateVersion')
        notBlockDBForCommands = [s.strip() for s in config.getString('MAIN', 'notBlockDBForCommands').split(',')]
        suffix = shop.get('shopId', '')
        self.modes = ['ANY', 'CS']
        if shop.get('shopId', '').endswith('All') and shop.get('shopId', '').startswith(shop.get('clusterId', '')):
            suffix = 'All'
            self.modes.append('CS_ALL')
        executor = Exec(host=host, port=port, user=user, password=password,
                database='%sdictionaries%s' % (shop.get('clusterId', ''), suffix),
                shopCode=shop.get('shopId', ''), loadDictStatusSender=loadDictStatusSender)
        Inserter.__init__(self, shop.get('shopId', ''), executor, notBlockDBForCommands)
        self.mysqlConnBlock = MySQLdb.connect(host=host, user=user, passwd=password, db='artixcs_struct', port=port)
        self.mysqlConnBlock.autocommit(True)
        self.dbForCheckVersion = '%sartixcs%s' % (shop.get('clusterId', ''), suffix)
        self.mysqlConnDBVersion = MySQLdb.connect(host=host, user=user, passwd=password, db=self.dbForCheckVersion, port=port)
        self.mysqlConnDBVersion.autocommit(True)
        if self.dbForCheckVersion not in self.dbVersionIsOK:
            self.dbVersionIsOK[self.dbForCheckVersion] = False

    def testDBVersion(self):
        if self.dbVersionIsOK.get(self.dbForCheckVersion, False):
            return True
        try:
            cur = self.mysqlConnDBVersion.cursor()
            cur.execute("select version from migrate_version where repository_id like 'artixcs%'")
            row = cur.fetchone()
            dbVersion = row[0] if row is not None else self.migrateVersion
            self.log.debug('migrateVersion fom DB = %s' % dbVersion)
            self.log.debug('migrateVersion fom config = %s' % self.migrateVersion)
            if dbVersion >= self.migrateVersion:
                self.dbVersionIsOK[self.dbForCheckVersion] = True
                return True
        except Exception as e:
            self.log.warning('%s' % e)
            return False
        finally:
            cur.close()
        return False

    def flagBlockSet(self):
        cur = self.mysqlConnBlock.cursor()
        self.log.info('Block storeId=%s' % self.shopCode)
        try:
            res = cur.execute("insert into extlocktable (storeId,lockedBy) values ( '%s', 'NES')" % self.shopCode)
            if res == 1:
                self.blocked = True
                return 1
        except:
            return 0
        finally:
            cur.close()
        return 0

    def testFlagBlockSet(self):
        cur = self.mysqlConnBlock.cursor()
        cur.execute("select * from extlocktable where storeId = '%s' and lockedBy = 'NES'" % self.shopCode)
        if cur.fetchone() is not None:
            cur.close()
            return True
        cur.close()
        return False

    def flagUnBlockSet(self):
        cur = self.mysqlConnBlock.cursor()
        self.log.info('UnBlock storeId=%s' % self.shopCode)
        cur.execute("delete from extlocktable where storeId = '%s' and lockedBy = 'NES'" % self.shopCode)
        cur.close()
        self.blocked = False
        if self.testFlagBlockSet():
            return 0
        return 1
    
    def getCommand(self, simpleCommands):
        return Command(self.executor, self.shopCode, simpleCommands=simpleCommands, modes=self.modes)
