# -*- coding: UTF-8 -*-

from nes_common.sql_operation import _Insert, _Update as Update, \
    _Select as Select, _Delete as Delete, _DeleteAll, _DeleteByDate as DeleteByDate, \
    _DeleteOverduePrices as DeleteOverduePrices, _DeleteBeforeDate as DeleteBeforeDate, \
    _DeleteAfterDate as DeleteAfterDate


class Insert(_Insert):
    """ MySQL запрос 'Insert' """

    def prepareStatement(self, context):
        if self.cacheKey in _Insert.cache:
            return _Insert.cache[self.cacheKey]
        context.tableName(self.table)  # вот в этом месте кэшируется имя таблицы для дальнейшего определения задачи на выгрузку. Нельзя ее не вызывать
        fields = ['`%s`' % k for k in self.values.keys()]
        fieldsCount = len(fields)
        dupValue = ",".join(map(lambda f: f + " = VALUES(" + f + ")", fields)) + (
                ',`ownerStoreId` = NULL' if self.cacheOwnerStoreIdPresented.get(self.table, False) else '')
        dataObj = {
            'table': self.table,
            'fields': ','.join(fields),
            'dup': dupValue,
            'values': ','.join([self.replaceSymbol] * fieldsCount)
        }
        sql = 'INSERT IGNORE INTO %(table)s (%(fields)s) VALUES (%(values)s) ON DUPLICATE KEY UPDATE %(dup)s' % dataObj
        _Insert.cache[self.cacheKey] = sql
        return sql


class DeleteAll(_DeleteAll):
    """ Очистка таблицы """

    def execute(self, context):
        self.execSql(context, "DELETE FROM `%s`" % context.tableName(self.table))
        context.emptyTables[self.table] = True


class DeleteNotNestedItems(Delete):
    """ Удаление записей с указанным внешним ключём и не входящих в список исключений """

    def __init__(self, table, mainCodeField, mainCodeValue, nestedCodeField, nestedCodeIn, notExecMany=False):
        """ Удаление записей с указанным внешним ключём и не входящих в список исключений по указанной колонке
        Args:
            table (str): Таблица для удаления данных
            mainCodeField (str): Колонка с внешним ключём
            mainCodeValue (str|int|decimal): Значение внешнего ключа
            nestedCodeField (str): Колонка для проверки исключений
            nestedCodeIn (list[str|int|decimal]): Список исключений
            notExecMany (bool): Не применять групповое выполнение для этого запроса
        """
        self.mainCodeField = mainCodeField
        self.mainCodeValue = mainCodeValue
        self.nestedCodeField = nestedCodeField
        if nestedCodeIn:
            self.nestedCodeIn = nestedCodeIn
            Delete.__init__(self, table, [mainCodeField, nestedCodeField], [mainCodeValue, nestedCodeIn], notExecMany)
        else:
            self.nestedCodeIn = []
            Delete.__init__(self, table, mainCodeField, mainCodeValue, notExecMany)

    def __str__(self):
        return '[DELETE table=%s, mainCodeField=%s, mainCodeValue=%s, nestedCodeField=%s, nestedCodeIn=%s]' % (
            self.table, self.mainCodeField, self.mainCodeValue, self.nestedCodeField, self.nestedCodeIn)

    def prepareStatement(self, context):
        if self.cacheKey in Delete.cache:
            sql = Delete.cache[self.cacheKey]
        tableName = context.tableName(self.table)
        if self.nestedCodeIn:
            whereClause = 'WHERE `%s` = %s AND `%s` NOT IN %s' % (self.mainCodeField, self.replaceSymbol, self.nestedCodeField, self.replaceSymbol)
            Delete.cacheUnite[self.cacheKey] = False
        else:
            whereClause = self.formWhereClause(context, self.whereFields, self.whereValues)
            Delete.cacheUnite[self.cacheKey] = True
        sql = 'DELETE FROM `%s` %s' % (tableName, whereClause)
        Delete.cache[self.cacheKey] = sql
        return sql
