# -*- coding: UTF-8 -*-

import time
import nes_common.logger as logger
from datetime import datetime
from nes_command import Command


class Inserter(object):
    def __init__(self, shopCode, executor, notBlockDBForCommands=[]):
        self.shopCode = shopCode
        self.log = logger.getLogger('Shop', shopCode)
        self.executor = executor  # Exec(driver, server, user, password, database, shopCode) # Exec(host, port, user, password, database, shopCode)
        self.cmd = None
        self.VERSION = '4.4'
        self.startTime = None
        self.blocked = False
        self.notBlockDBForCommands = notBlockDBForCommands

    def flagBlockSet(self):
        return 1
    
    def testFlagBlockSet(self):
        return False
    
    def testDBVersion(self):
        return True

    def flagUnBlockSet(self):
        return 1
    
    def getCommand(self, simpleCommands):
        return Command(self.executor, self.shopCode, simpleCommands=simpleCommands, modes=['ANY'])
    
    def blockSet(self):
        while self.flagBlockSet() != 1:
            self.log.info('Database busy')
            if self.testFlagBlockSet():
                self.flagUnBlockSet()
            time.sleep(5)
    
    def start(self, messageId):
        self.startTime = datetime.now()
        self.log.info('Artix NES uploader version %s started at %s' % (self.VERSION, self.startTime))
        while not self.testDBVersion():
            self.log.info('Database version is outdated')
            time.sleep(30)
        if not self.notBlockDBForCommands:
            self.blockSet()
        try:
            self.executor.startUpload(messageId)
            simpleCommands = set(['addClient', 'addCard'])
            self.cmd = self.getCommand(simpleCommands)
            return True
        except Exception as e:
            self.log.error(e)
            if self.flagUnBlockSet() != 1:
                self.log.info('Database not unblock')
        return False

    def addCount(self, dictCountCommands, commandName):
        for cmdPrefix in dictCountCommands.keys():
            if commandName.startswith(cmdPrefix):
                dictCountCommands[cmdPrefix] += 1
                return
            elif commandName.startswith('replace'):
                dictCountCommands['add'] += 1
                return

    def add_package(self, dictObjects):
        tableNamesWithError = set()
        countGoodCommands = {'add': 0, 'delete': 0, 'clear': 0, 'update': 0}
        countBadCommands = {'add': 0, 'delete': 0, 'clear': 0, 'update': 0}
        for dictObject in dictObjects:
            if not self.blocked and dictObject['command'] not in self.notBlockDBForCommands:
                self.blockSet()
            self.executor.updateProgress()
            self.log.debug('dictObject = %s ' % dictObject)
            method = self.cmd.dispatch(dictObject)
            if method:
                res, tables = method(dictObject)
                if res:
                    self.addCount(countGoodCommands, dictObject['command'])
                else:
                    self.addCount(countBadCommands, dictObject['command'])
                    if isinstance(tables, list):
                        tableNamesWithError = tableNamesWithError.union(set(tables))
            else:
                self.log.warn('Command not found. Object = %s' % dictObject)
                self.addCount(countBadCommands, dictObject['command'])
        return tableNamesWithError, countGoodCommands, countBadCommands

    def stop(self):
        try:
            self.executor.finishUpload()
            while self.flagUnBlockSet() != 1:
                self.log.info('Database not unblocked')
                time.sleep(5)
            endTime = datetime.now()
            self.log.info('Artix NES uploader finished at %s' % endTime)
            self.log.info("Total execution time: %s" % (endTime - self.startTime))
            return True
        except Exception as e:
            self.log.error("Can't finish upload and unblock DB. %s" % e)
        return False
