# -*- coding: utf-8 -*-

import mysql.simplesql as sql

tables = ['tmc', 'tprices', 'tmcsalerestrict']


def barcodes(inventItem):
    return inventItem.get('barcodes', [])


def additionalPrices(inventOrBarcode):
    return inventOrBarcode.get('additionalprices', [])


def timeRestrictions(inventOrBarcode):
    return inventOrBarcode.get('sellrestrictperiods', [])


def quantModeForOptions(options):
    quantMode = 0
    if options.get("quantityoptions", {}).get("enabledefaultquantity", 1) == 1:
        quantMode |= 1
    if options.get("quantityoptions", {}).get("enablequantitymanual", 1) == 1:
        quantMode |= 2
    if options.get("quantityoptions", {}).get("enablequantitybarcode", 1) == 1:
        quantMode |= 4
    if options.get("quantityoptions", {}).get("enablequantityscales", 1) == 1:
        quantMode |= 8
    if options.get("quantityoptions", {}).get("enablequantitylimit", 0) == 1:
        quantMode |= 16
    if options.get("quantityoptions", {}).get("requirequantitymanual", 0) == 1:
        quantMode |= 32
    if options.get("quantityoptions", {}).get("requirequantityscales", 0) == 1:
        quantMode |= 64
    if options.get("quantityoptions", {}).get("requirequantitybarcode", 0) == 1:
        quantMode |= 128
    if options.get("quantityoptions", {}).get("enabledocumentquantitylimit", 0) == 1:
        quantMode |= 256
    if options.get("quantityoptions", {}).get("enabledocumentquantitylimit", 0) == 2:
        quantMode |= 512
    if options.get("quantityoptions", {}).get("autogetquantityfromscales", 0) == 1:
        quantMode |= 1024
    return quantMode


def bcodeModeForOptions(options):
    bcodeMode = 0
    if options.get("inventitemoptions", {}).get("enablebarcodemanual", 1) == 1:
        bcodeMode |= 1
    if options.get("inventitemoptions", {}).get("enablebarcodescanner", 1) == 1:
        bcodeMode |= 2
    return bcodeMode


def opModeForOptions(options):
    opMode = 0
    if options.get("inventitemoptions", {}).get("disableinventsale", 0) == 1:
        opMode |= 1
    if options.get("inventitemoptions", {}).get("disableinventback", 0) == 1:
        opMode |= 2
    if options.get("inventitemoptions", {}).get("disableinventshow", 0) == 1:
        opMode |= 4
    if options.get("inventitemoptions", {}).get("disablebackinsale", 0) == 1:
        opMode |= 8
    if options.get("inventitemoptions", {}).get("visualverify", 0) == 1:
        opMode |= 16
    if options.get("inventitemoptions", {}).get("ageverify", 0) == 1:
        opMode |= 32
    if options.get("inventitemoptions", {}).get("requiresalerestrict", 0) == 1:
        opMode |= 64
    return opMode


def deptModeForOptions(options):
    deptMode = 0
    if options.get("inventitemoptions", {}).get("enabledepartmentmanual", 1) == 1:
        deptMode |= 1
    if options.get("inventitemoptions", {}).get("requiredepartmentmanual", 0) == 1:
        deptMode |= 2
    return deptMode


def priceModeForOptions(options):
    priceMode = 0
    if options.get("discoptions", {}).get("enablediscshemeinput", 1) == 1:
        priceMode |= 1
    if options.get("discoptions", {}).get("requirediscshemeinput", 0) == 1:
        priceMode |= 2
    if options.get("priceoptions", {}).get("requireselectprice", 0) == 1:
        priceMode |= 4
    if options.get("priceoptions", {}).get("requiredeferredprice", 0) == 1:
        priceMode |= 8
    return priceMode


def extractTmcPropertiesFromInventItem(inventItem):
    vatcodes = inventItem.get('lvatcode', [0, 0, 0, 0, 0])
    taxGroupCode = inventItem.get('taxgroupcode', None)
    vatcodes += [0] * (5 - len(vatcodes))
        
    if taxGroupCode:
        for n in range(5):
            vatcodes[n] = taxGroupCode * 100 + n + 1

    return sql.Insert(table='tmc', values={
        'vatcode1': vatcodes[0],
        'vatcode2': vatcodes[1],
        'vatcode3': vatcodes[2],
        'vatcode4': vatcodes[3],
        'vatcode5': vatcodes[4],
        'dcode': inventItem.get('deptcode', 0),
        'name': inventItem.get('name', ''),
        'articul': inventItem.get('articul', ''),
        'cquant': inventItem.get('cquant', 1.0),
        'measure': inventItem.get('measurecode', 0),
        'pricetype': inventItem.get('discautoscheme', 0),
        'price': inventItem.get('price', 0.0),
        'minprice': inventItem.get('minprice', 0.0),
        'valcode': 0,
        'quantdefault': inventItem.get('defaultquantity', 1.0),
        'quantlimit': inventItem.get('options', {}).get('quantityoptions', {}).get('quantitylimit', 0),
        'ostat': 0,
        'links': 0,
        'quant_mode': quantModeForOptions(inventItem.get('options', {})),
        'bcode_mode': bcodeModeForOptions(inventItem.get('options', {})),
        'op_mode': opModeForOptions(inventItem.get('options', {})),
        'dept_mode': deptModeForOptions(inventItem.get('options', {})),
        'price_mode': priceModeForOptions(inventItem.get('options', {})),
        'groupcode': inventItem.get('inventgroup', None),
        'extendetoptions': inventItem.get('extendedoptions', None),
        'tara_mode': inventItem.get('taramode', None),
        'tara_default': '0',
        'unit_weight': inventItem.get('taracapacity', None),
        'remain': inventItem.get('remain', None),
        'aspectschemecode': inventItem.get('aspectschemecode', None),
        'aspectvaluesetcode': inventItem.get('aspectvaluesetcode', None),
        'aspectusecase': inventItem.get('aspectusecase', None),
        'aspectselectionrule': inventItem.get('aspectselectionrule', None),
        'remaindate': inventItem.get('remaindate', None),
        'documentquantlimit': inventItem.get('options', {}).get('quantityoptions', {}).get('documentquantlimit', 0),
        'age': inventItem.get('age', None),
        'alcoholpercent': inventItem.get('alcoholpercent', None),
    })


def tmcWithInventCode(inventItem):
    tmc = extractTmcPropertiesFromInventItem(inventItem)
    tmc['code'] = inventItem.get('inventcode', None)
    tmc['bcode'] = inventItem.get('inventcode', None)
    return tmc


def tmcWithOwnBarcode(inventItem):
    barcode = inventItem.get('barcode', None)
    code = inventItem.get('inventcode', None)
    if barcode is not None and barcode != code:
        tmc = extractTmcPropertiesFromInventItem(inventItem)
        tmc['code'] = code
        tmc['bcode'] = barcode
        return tmc
    return None


def propagateBarcode(inventItem, barcodeItem):
    result = inventItem.copy()
    for key in barcodeItem.keys():
        result[key] = barcodeItem[key]
    return result


def tmcWithBarcodes(inventItem):
    for bcode in barcodes(inventItem):
        mixedInvent = propagateBarcode(inventItem, bcode)
        tmc = tmcWithOwnBarcode(mixedInvent)
        if tmc:
            yield tmc
        

def tpricesForInventItem(inventItem):
    tmpBcodes = []
    for bcode in [inventItem.get('inventcode', None), inventItem.get('barcode', None)]:
        if tmpBcodes.count(bcode) == 0:
            tmpBcodes.append(bcode)
            if bcode is None:
                continue
            for additionalPrice in additionalPrices(inventItem):
                index = additionalPrice.get('pricecode', None)
                if index is None:
                    continue
                tprices = sql.Insert(table='tprices', values={
                    'tindex': index,
                    'bcode': bcode,
                    'price': additionalPrice.get('price', 0.0),
                    'name': additionalPrice.get('name', '')
                })
                yield tprices


def tpricesForBarcode(inventItem, barcode):
    for additionalPrice in additionalPrices(barcode):
        index = additionalPrice.get('pricecode', None)
        bcode = barcode.get('barcode', None)
        tprices = sql.Insert(table='tprices', values={
            'tindex': index,
            'bcode': bcode,
            'price': additionalPrice.get('price', 0.0),
            'name': additionalPrice.get('name', '')
        })
        if index and bcode:
            yield tprices


def tpricesForBarcodes(inventItem):
    for barcode in barcodes(inventItem):
        for t in tpricesForBarcode(inventItem, barcode):
            yield t


def timeRestrictionsForItem(inventItemOrBarcode):
    for r in timeRestrictions(inventItemOrBarcode):
        yield sql.Insert(table='tmcsalerestrict', values={
            'dateend': r.get('dateend', None),
            'datebeg': r.get('datestart', None),
            'daysend': r.get('dayend', None),
            'daysbeg': r.get('daystart', None),
            'timeend': r.get('timeend', None),
            'timebeg': r.get('timestart', None)
        })


def timeRestrictionsForInventItem(inventItem):
    for r in timeRestrictionsForItem(inventItem):
        r['tmccode'] = inventItem.get('inventcode', None)
        yield r


def timeRestrictionsForBarcode(inventItem, barcode):
    return []
#    for r in timeRestrictionsForItem(inventItem):
#        r['tmccode'] = inventItem.get('inventcode', None)
#        yield r


def timeRestrictionsForBarcodes(inventItem):
    for barcode in barcodes(inventItem):
        for r in timeRestrictionsForBarcode(inventItem, barcode):
            yield r


def insertInventItem(inventItem):
    t = tmcWithInventCode(inventItem)
    if t:
        yield t
    for t in tmcWithBarcodes(inventItem):
        yield t
    t = tmcWithOwnBarcode(inventItem)
    if t:
        yield t
    for p in tpricesForInventItem(inventItem):
        yield p
    for p in tpricesForBarcodes(inventItem):
        yield p
    for r in timeRestrictionsForInventItem(inventItem):
        yield r
    for r in timeRestrictionsForBarcodes(inventItem):
        yield r


def deleteInventItemByCode(code):
    barcodes = sql.Select(table='tmc', fields=['bcode'], whereFields=['code'], whereValues=[code])
    yield sql.Delete(table='tprices', whereFields=['bcode'], whereValues=barcodes)
    yield sql.Delete(table='tprices', whereFields=['bcode'], whereValues=[code])
    yield sql.Delete(table='tmcsalerestrict', whereFields=['tmccode'], whereValues=[code])
    yield sql.Delete(table='tmc', whereFields=['code'], whereValues=[code])


def deleteInventItem(inventItem):
    code = inventItem.get('inventcode', None)
    barcodes = filter(lambda x: x is not None, map(lambda x: x.get('barcode', None), inventItem.get('barcodes', [])))
    for k in deleteInventItemByCode(code):
        yield k
    # TODO: НУЖНО ЕЩЁ ИЗ ТОВАРА ШК ГРОХНУТЬ!!!
    for barcode in barcodes:
        yield sql.Delete(table='tmc', whereFields=['bcode'], whereValues=[barcode])


def clearInventory():
    yield sql.DeleteAll(table='tmc')
    yield sql.DeleteAll(table='tmcsalerestrict')
    yield sql.DeleteAll(table='tprices')


def updateInventItem(inventItem):
    fields = []
    values = []
    
    if inventItem.get('taramode'):
        fields.append('tara_mode')
        values.append(inventItem.get('taramode'))
        
    if inventItem.get('taracapacity'):
        fields.append('unit_weight')
        values.append(inventItem.get('taracapacity'))
    
    if inventItem.get('sellrestrictperiods'):
        r = inventItem.get('sellrestrictperiods')
        yield sql.Insert(table='tmcsalerestrict', values={
            'dateend': r.get('dateend', None),
            'datebeg': r.get('datestart', None),
            'daysend': r.get('dayend', None),
            'daysbeg': r.get('daystart', None),
            'timeend': r.get('timeend', None),
            'timebeg': r.get('timestart', None),
            'tmccode': inventItem.get('inventcode', None)
        })
    
    if fields:
        yield sql.Update(table='tmc', fields=fields, values=values, whereField=['bcode'],
                whereValue=[inventItem.get('inventcode', 0)])
